@@ -18,26 +18,23 @@
 """A CLI for jupyterlab-requirements: Horus."""
 
 import logging
-import os
 import json
 import sys
 import subprocess
-import yaml
+import yaml  # type: ignore
 import click
 import typing
 
 from typing import Optional
 from pathlib import Path
-from rich.console import Console
-from rich.table import Table
 
 from thoth.python import Pipfile, PipfileLock
 from thamos.config import _Configuration
 from thamos.discover import discover_python_version
+from thamos.cli import _parse_labels
 
 from jupyterlab_requirements import __version__
 
-from jupyterlab_requirements.dependency_management import _EMOJI
 from jupyterlab_requirements.dependency_management import create_pipfile_from_packages
 from jupyterlab_requirements.dependency_management import gather_libraries
 from jupyterlab_requirements.dependency_management import get_packages
@@ -46,18 +43,21 @@ from jupyterlab_requirements.dependency_management import horus_check_metadata_c
 from jupyterlab_requirements.dependency_management import horus_delete_kernel
 from jupyterlab_requirements.dependency_management import horus_extract_command
 from jupyterlab_requirements.dependency_management import horus_lock_command
+from jupyterlab_requirements.dependency_management import horus_log_command
 from jupyterlab_requirements.dependency_management import horus_requirements_command
 from jupyterlab_requirements.dependency_management import horus_set_kernel_command
 from jupyterlab_requirements.dependency_management import horus_show_command
 from jupyterlab_requirements.dependency_management import horus_list_kernels
 from jupyterlab_requirements.dependency_management import load_files
+from jupyterlab_requirements.dependency_management import print_report
 from jupyterlab_requirements.dependency_management import save_notebook_content
+from jupyterlab_requirements.dependency_management import verify_gathered_libraries
 
 
 _LOGGER = logging.getLogger("thoth.jupyterlab_requirements.cli")
 
 
-def _print_version(ctx: click.Context, _, value: str):
+def _print_version(ctx: click.Context, value: str) -> None:
     """Print version and exit."""
     if not value or ctx.resilient_parsing:
         return
@@ -76,10 +76,10 @@ def _print_version(ctx: click.Context, _, value: str):
     help="Be verbose about what's going on.",
 )
 def cli(
-    ctx: click.Context = None,
+    ctx: Optional[click.Context] = None,
     verbose: bool = False,
     output: Optional[str] = None,
-):
+) -> None:
     """Horus: CLI for jupyterlab-requirements."""
     if verbose:
         _LOGGER.setLevel(logging.DEBUG)
@@ -91,7 +91,7 @@ def cli(
 @cli.command("version")
 @click.pass_context
 @click.option("--json", "-j", "json_output", is_flag=True, help="Print output in JSON format.")
-def version(ctx: click.Context, json_output: bool = False):
+def version(ctx: click.Context, json_output: bool = False) -> None:
     """Print Horus, Thamos and Thoth version and exit."""
     click.echo(f"Horus (jupyterlab-requirements CLI) version: {__version__}")
 
@@ -144,7 +144,7 @@ def extract(
     thoth_config: bool = False,
     use_overlay: bool = False,
     force: bool = False,
-):
+) -> None:
     """Extract dependencies from notebook metadata.
 
     Examples:
@@ -197,7 +197,7 @@ def show(
     pipfile: bool = False,
     pipfile_lock: bool = False,
     thoth_config: bool = False,
-):
+) -> None:
     """Show dependencies from notebook metadata.
 
     Examples:
@@ -219,6 +219,9 @@ def show(
     else:
         click.echo(f"Resolution engine identified: {results['dependency_resolution_engine']!s}")
 
+    if results["thoth_analysis_id"]:
+        click.echo(f"Thoth analysis ID: {results['thoth_analysis_id']!s}")
+
     if results["pipfile"]:
         click.echo(results["pipfile"])
 
@@ -286,7 +289,7 @@ def save(
     pipfile_lock: bool = False,
     thoth_config: bool = False,
     force: bool = False,
-):
+) -> None:
     """Save dependencies in notebook metadata.
 
     Examples:
@@ -315,7 +318,7 @@ def save(
         pipfile_string, pipfile_lock_string = load_files(base_path=save_files_path)
 
         pipfile_ = Pipfile.from_string(pipfile_string)
-        pipfile_lock_ = PipfileLock.from_string(pipfile_content=pipfile_lock_string, pipfile=pipfile_)
+        pipfile_lock_ = PipfileLock.from_string(pipfile_content=str(pipfile_lock_string), pipfile=pipfile_)
 
     if pipfile or save_all:
         if "requirements" in notebook_metadata and not force:
@@ -348,8 +351,8 @@ def save(
     if resolution_engine == "thoth":
 
         if thoth_config or save_all:
-            config = _Configuration()
-            config.load_config_from_file(config_path=Path(save_files_path).joinpath(".thoth.yaml"))
+            config = _Configuration()  # type: ignore
+            config.load_config_from_file(config_path=str(Path(save_files_path).joinpath(".thoth.yaml")))
 
             if "thoth_config" in notebook_metadata and not force:
                 raise FileExistsError(
@@ -394,7 +397,9 @@ def save(
     is_flag=True,
     help="Force actions for creation of Pipfile.",
 )
-def discover(ctx: click.Context, path: str, store_files_path: str, show_only: bool = False, force: bool = False):
+def discover(
+    ctx: click.Context, path: str, store_files_path: str, show_only: bool = False, force: bool = False
+) -> None:
     """Discover dependencies from notebook content.
 
     Examples:
@@ -404,7 +409,9 @@ def discover(ctx: click.Context, path: str, store_files_path: str, show_only: bo
 
         horus discover [YOUR_NOTEBOOK].ipynb --force
     """
-    packages = gather_libraries(notebook_path=path)
+    gathered_libraries = gather_libraries(notebook_path=path)
+    verified_libraries = verify_gathered_libraries(gathered_libraries=gathered_libraries)
+    packages = [package["package_name"] for package in verified_libraries]
 
     if packages:
         click.echo(f"Thoth invectio libraries gathered: {json.dumps(packages)}")
@@ -429,7 +436,7 @@ def discover(ctx: click.Context, path: str, store_files_path: str, show_only: bo
                 "Use --force to overwrite existing content or --show-only to visualize the Pipfile."
             )
         else:
-            pipfile.to_file(path=pipfile_path)
+            pipfile.to_file(path=str(pipfile_path))
 
     ctx.exit(0)
 
@@ -467,34 +474,10 @@ def check(ctx: click.Context, path: str, output_format: str) -> None:
         sys.stdout.write("\n")
 
     elif output_format == "table":
-        table = Table()
-
-        header = set()
-        for item in result:
-            for key in item.keys():
-                header.add(key)
-
-        header_sorted = sorted(header)
-        for element in header_sorted:
-            table.add_column(
-                element.replace("_", " ").capitalize(),
-                style="cyan",
-                overflow="fold",
-            )
-
-        for item in result:
-            row = []
-            for key in header_sorted:
-                entry = item.get(key)
-                if not bool(int(os.getenv("JUPYTERLAB_REQUIREMENTS_NO_EMOJI", 0))) and isinstance(entry, str):
-                    entry = _EMOJI.get(entry, entry)
-
-                row.append(entry if entry is not None else "-")
-
-            table.add_row(*row)
-
-        console = Console()
-        console.print(table, justify="center")
+        print_report(
+            result,
+            title="Horus check results",
+        )
 
     sys.exit(1 if any(item.get("type") == "ERROR" for item in result) else 0)
 
@@ -559,32 +542,10 @@ def list_kernels(ctx: click.Context) -> None:
                 }
             )
 
-    table = Table()
-
-    header = set()
-    for item in result:
-        for key in item.keys():
-            header.add(key)
-
-    header_sorted = sorted(header)
-    for element in header_sorted:
-        table.add_column(
-            element.replace("_", " ").capitalize(),
-            style="cyan",
-            overflow="fold",
-        )
-
-    for item in result:
-        row = []
-        for key in header_sorted:
-            entry = item.get(key)
-            row.append(entry if entry is not None else "-")
-
-        table.add_row(*row)
-
-    console = Console()
-
-    console.print(table, justify="center")
+    print_report(
+        result,
+        title="Kernels available",
+    )
 
 
 @cli.command("delete-kernel")
@@ -643,32 +604,10 @@ def check_kernel(ctx: click.Context, kernel_name: str) -> None:
 
         result.append({"package name": package_name, "package version": packages[package_name]})
 
-    table = Table()
-
-    header = set()
-    for item in result:
-        for key in item.keys():
-            header.add(key)
-
-    header_sorted = sorted(header)
-    for element in header_sorted:
-        table.add_column(
-            element.replace("_", " ").capitalize(),
-            style="cyan",
-            overflow="fold",
-        )
-
-    for item in result:
-        row = []
-        for key in header_sorted:
-            entry = item.get(key)
-            row.append(entry if entry is not None else "-")
-
-        table.add_row(*row)
-
-    console = Console()
-
-    console.print(table, justify="center")
+    return print_report(
+        result,
+        title=f"{kernel_name} kernel packages",
+    )
 
     ctx.exit(0)
 
@@ -809,6 +748,15 @@ def requirements(
     required=False,
     help="Python version for Thoth advise request.",
 )
+@click.option(
+    "--labels",
+    "-l",
+    type=str,
+    metavar="KEY1=VALUE1,KEY2=VALUE2",
+    default=None,
+    show_default=True,
+    help="Labels used to label the request.",
+)
 def lock(
     ctx: click.Context,
     path: str,
@@ -821,6 +769,7 @@ def lock(
     os_name: Optional[str] = None,
     os_version: Optional[str] = None,
     python_version: Optional[str] = None,
+    labels: Optional[str] = None,
 ) -> None:
     """Lock requirements in notebook metadata.
 
@@ -845,6 +794,7 @@ def lock(
         os_name=os_name,
         os_version=os_version,
         python_version=python_version,
+        labels=_parse_labels(labels),
     )
 
     if results["kernel_name"] == "python3":
@@ -857,7 +807,18 @@ def lock(
         ctx.exit(1)
     else:
         if resolution_engine == "thoth":
-            click.echo("Thoth successfully resolved your stack.")
+            if lock_results["stack_info"]:
+                print_report(
+                    lock_results["stack_info"],
+                    title="Application stack guidance",
+                )
+
+            # Print report of the best one - thus index zero.
+            if lock_results["justification"]:
+                print_report(
+                    lock_results["justification"],
+                    title="Recommended stack report",
+                )
         else:
             click.echo("Pipenv successfully resolved your stack.")
 
@@ -867,4 +828,23 @@ def lock(
         )
 
 
+@cli.command("log")
+@click.pass_context
+@click.argument("path")
+def log(
+    ctx: click.Context,
+    path: str,
+) -> None:
+    """Get log of finished analysis (only for Thoth)."""
+    try:
+        log_str = horus_log_command(notebook_path=path)
+
+        if log_str:
+            click.echo(log_str)
+    except Exception as e:
+        click.echo(e)
+
+    ctx.exit(1)
+
+
 __name__ == "__main__" and cli()
