@@ -1,5 +1,5 @@
 # jupyterlab-requirements
-# Copyright(C) 2020 Francesco Murdaca
+# Copyright(C) 2020, 2021 Francesco Murdaca
 #
 # This program is free software: you can redistribute it and / or modify
 # it under the terms of the GNU General Public License as published by
@@ -40,12 +40,15 @@ from .lib import horus_check_metadata_content
 from .lib import horus_delete_kernel
 from .lib import horus_extract_command
 from .lib import horus_lock_command
+from .lib import horus_log_command
 from .lib import horus_requirements_command
 from .lib import horus_set_kernel_command
 from .lib import horus_show_command
 from .lib import horus_list_kernels
 from .lib import load_files
+from .lib import print_report
 from .lib import save_notebook_content
+from .lib import verify_gathered_libraries
 
 from .magic_commands import HorusMagics
 
@@ -73,11 +76,14 @@ __all__ = [
     "horus_extract_command",
     "horus_set_kernel_command",
     "horus_lock_command",
+    "horus_log_command",
     "horus_requirements_command",
     "horus_show_command",
     "horus_list_kernels",
     "load_files",
+    "print_report",
     "save_notebook_content",
+    "verify_gathered_libraries",
     "HorusMagics",
     "_EMOJI",
 ]
