@@ -25,10 +25,11 @@ import typing
 import tempfile
 import json
 import sys
-import yaml
+import yaml  # type: ignore
 import invectio
 import distutils.sysconfig as sysconfig
 
+from subprocess import CompletedProcess
 from virtualenv import cli_run
 from pathlib import Path
 
@@ -43,6 +44,8 @@ from thoth.python import Source, PackageVersion
 from thoth.common import ThothAdviserIntegrationEnum
 
 from thamos.lib import advise_using_config, _get_origin
+from thamos.lib import get_package_from_imported_packages
+from thamos.lib import get_log
 from thamos.config import _Configuration
 from thamos.discover import discover_python_version
 
@@ -56,7 +59,7 @@ _EMOJI = {
 }
 
 
-def _print_report(report: dict, title: typing.Optional[str] = None):
+def print_report(report: typing.List[typing.Dict[str, typing.Any]], title: str = "") -> None:
     """Print reasoning to user."""
     console = Console()
     table = Table(
@@ -76,8 +79,8 @@ def _print_report(report: dict, title: typing.Optional[str] = None):
     header = header - to_remove
 
     header_list = sorted(header)
-    for item in header_list:
-        table.add_column(item.replace("_", " ").capitalize(), style="cyan", overflow="fold")
+    for item_ in header_list:
+        table.add_column(item_.replace("_", " ").capitalize(), style="cyan", overflow="fold")
 
     for item in report:
         row = []
@@ -100,7 +103,7 @@ def _print_report(report: dict, title: typing.Optional[str] = None):
     console.print(table, justify="center")
 
 
-def get_notebook_content(notebook_path: str, py_format: bool = False):
+def get_notebook_content(notebook_path: str, py_format: bool = False) -> typing.Any:
     """Get JSON of the notebook content."""
     actual_path = Path(notebook_path)
 
@@ -129,22 +132,34 @@ def get_notebook_content(notebook_path: str, py_format: bool = False):
         return notebook_content_py
 
 
-def horus_check_metadata_content(notebook_metadata: dict, is_cli: bool = True) -> list:
+def horus_check_metadata_content(
+    notebook_metadata: typing.Dict[str, typing.Any], is_cli: bool = True
+) -> typing.List[typing.Dict[str, typing.Any]]:
     """Check the metadata of notebook for dependencies."""
     result = []
 
     if notebook_metadata.get("language_info"):
         language = notebook_metadata["language_info"]["name"]
 
-        if language and language != "python":
-            result.append(
-                {
-                    "message": "Only python programming language is supported.",
-                    "type": "ERROR",
-                }
-            )
+        if language:
+            if language != "python":
+                result.append(
+                    {
+                        "key": "programming_language",
+                        "message": "Only python is supported.",
+                        "type": "ERROR",
+                    }
+                )
+                return result
 
-            return result
+            else:
+                result.append(
+                    {
+                        "key": "programming_language",
+                        "message": f"{language}",
+                        "type": "INFO",
+                    }
+                )
 
     kernelspec = notebook_metadata.get("kernelspec")
 
@@ -156,7 +171,8 @@ def horus_check_metadata_content(notebook_metadata: dict, is_cli: bool = True) -
 
     result.append(
         {
-            "message": f"kernel name: {kernel_name}",
+            "key": "kernel_name",
+            "message": kernel_name,
             "type": "INFO",
         }
     )
@@ -169,7 +185,8 @@ def horus_check_metadata_content(notebook_metadata: dict, is_cli: bool = True) -
 
         result.append(
             {
-                "message": "dependency_resolution_engine key is not present in notebook metadata. "
+                "key": "dependency_resolution_engine",
+                "message": "key is not present in notebook metadata. "
                 f"It will be set after creating Pipfile and running {command}",
                 "type": "WARNING",
             }
@@ -177,7 +194,8 @@ def horus_check_metadata_content(notebook_metadata: dict, is_cli: bool = True) -
     else:
         result.append(
             {
-                "message": f"dependency resolution engine: {notebook_metadata['dependency_resolution_engine']}",
+                "key": "dependency_resolution_engine",
+                "message": f"{notebook_metadata['dependency_resolution_engine']}",
                 "type": "INFO",
             }
         )
@@ -185,7 +203,7 @@ def horus_check_metadata_content(notebook_metadata: dict, is_cli: bool = True) -
     resolution_engine = notebook_metadata.get("dependency_resolution_engine")
 
     if resolution_engine == "thoth":
-        for thoth_specific_key in ["thoth_config"]:
+        for thoth_specific_key in ["thoth_config", "thoth_analysis_id"]:
             if thoth_specific_key not in notebook_metadata.keys():
 
                 if is_cli:
@@ -195,18 +213,29 @@ def horus_check_metadata_content(notebook_metadata: dict, is_cli: bool = True) -
 
                 result.append(
                     {
-                        "message": f"{thoth_specific_key} key is not present in notebook metadata. "
-                        f"You can run {command}.",
+                        "key": thoth_specific_key,
+                        "message": "key is not present in notebook metadata. " f"You can run {command}.",
                         "type": "ERROR",
                     }
                 )
             else:
-                result.append(
-                    {
-                        "message": f"{thoth_specific_key} key is present in notebook metadata.",
-                        "type": "INFO",
-                    }
-                )
+                if thoth_specific_key == "thoth_analysis_id":
+                    thoth_analysis_id = notebook_metadata.get("thoth_analysis_id")
+                    result.append(
+                        {
+                            "key": thoth_specific_key,
+                            "message": f"{thoth_analysis_id}",
+                            "type": "INFO",
+                        }
+                    )
+                else:
+                    result.append(
+                        {
+                            "key": thoth_specific_key,
+                            "message": "key is present in notebook metadata.",
+                            "type": "INFO",
+                        }
+                    )
 
     for mandatory_key in ["requirements"]:
         if mandatory_key not in notebook_metadata.keys():
@@ -218,7 +247,8 @@ def horus_check_metadata_content(notebook_metadata: dict, is_cli: bool = True) -
 
             result.append(
                 {
-                    "message": f"{mandatory_key} key is not present in notebook metadata. "
+                    "key": mandatory_key,
+                    "message": "key is not present in notebook metadata. "
                     f"You can run {command} with its options to set them.",
                     "type": "ERROR",
                 }
@@ -226,7 +256,8 @@ def horus_check_metadata_content(notebook_metadata: dict, is_cli: bool = True) -
         else:
             result.append(
                 {
-                    "message": f"{mandatory_key} key is present in notebook metadata.",
+                    "key": mandatory_key,
+                    "message": "key is present in notebook metadata.",
                     "type": "INFO",
                 }
             )
@@ -240,7 +271,8 @@ def horus_check_metadata_content(notebook_metadata: dict, is_cli: bool = True) -
 
             result.append(
                 {
-                    "message": f"{mandatory_key} key is not present in notebook metadata. "
+                    "key": mandatory_key,
+                    "message": f"key is not present in notebook metadata. "
                     f"You can run {command} if Pipfile already exists.",
                     "type": "ERROR",
                 }
@@ -249,7 +281,8 @@ def horus_check_metadata_content(notebook_metadata: dict, is_cli: bool = True) -
         else:
             result.append(
                 {
-                    "message": f"{mandatory_key} key is present in notebook metadata.",
+                    "key": mandatory_key,
+                    "message": "key is present in notebook metadata.",
                     "type": "INFO",
                 }
             )
@@ -267,6 +300,7 @@ def horus_check_metadata_content(notebook_metadata: dict, is_cli: bool = True) -
 
             result.append(
                 {
+                    "key": "requirements_hash_match",
                     "message": f"Pipfile hash stated in Pipfile.lock {project.pipfile_lock.meta.hash['sha256'][:6]} "
                     f"does not correspond to Pipfile hash {project.pipfile.hash()['sha256'][:6]} - was Pipfile "
                     f"adjusted? Then you should run {command}.",
@@ -276,6 +310,7 @@ def horus_check_metadata_content(notebook_metadata: dict, is_cli: bool = True) -
         else:
             result.append(
                 {
+                    "key": "requirements_hash_match",
                     "message": f"Pipfile hash stated in Pipfile.lock {project.pipfile_lock.meta.hash['sha256'][:6]} "
                     f"correspond to Pipfile hash {project.pipfile.hash()['sha256'][:6]}.",
                     "type": "INFO",
@@ -294,10 +329,11 @@ def horus_check_metadata_content(notebook_metadata: dict, is_cli: bool = True) -
                 else:
                     break
 
-        if check == len([p for p in notebook_packages]):
+        if check == len([p.name for p in notebook_packages]):
             result.append(
                 {
-                    "message": f"kernel {kernel_name} matches packages(name,version,PyPI index) "
+                    "key": "kernel_check",
+                    "message": f"kernel {kernel_name} matches packages (name,version,PyPI index) "
                     "from requirements lock in your notebook metadata.",
                     "type": "WARNING",
                 }
@@ -310,7 +346,8 @@ def horus_check_metadata_content(notebook_metadata: dict, is_cli: bool = True) -
 
             result.append(
                 {
-                    "message": f"kernel {kernel_name} selected does not match your dependencies. "
+                    "key": "kernel_check",
+                    "message": f"kernel {kernel_name} does not match your dependencies. \n"
                     f"Please run command {command} to create kernel for your notebook.",
                     "type": "WARNING",
                 }
@@ -368,7 +405,9 @@ def install_packages(
     )
 
 
-def get_packages(kernel_name: str, kernels_path: Path = Path.home().joinpath(".local/share/thoth/kernels")) -> dict:
+def get_packages(
+    kernel_name: str, kernels_path: Path = Path.home().joinpath(".local/share/thoth/kernels")
+) -> typing.Dict[str, str]:
     """Get packages in the virtualenv (pip list)."""
     _LOGGER.info(f"kernel_name selected: {kernel_name}")
 
@@ -451,13 +490,15 @@ def horus_list_kernels(kernels_path: Path = Path.home().joinpath(".local/share/t
     except Exception as e:
         _LOGGER.error(f"Kernel list could not be obtained: {e}")
 
-    # Delete folder from host
+    # Parse json output from command
     kernels = [k for k in json.loads(command_output.stdout.decode("utf-8"))["kernelspecs"]]
 
     return kernels
 
 
-def horus_delete_kernel(kernel_name: str, kernels_path: Path = Path.home().joinpath(".local/share/thoth/kernels")):
+def horus_delete_kernel(
+    kernel_name: str, kernels_path: Path = Path.home().joinpath(".local/share/thoth/kernels")
+) -> CompletedProcess:  # type: ignore
     """Delete kernel from host."""
     # Delete jupyter kernel
     try:
@@ -483,9 +524,63 @@ def horus_delete_kernel(kernel_name: str, kernels_path: Path = Path.home().joinp
     return command_output
 
 
+def verify_gathered_libraries(
+    gathered_libraries: typing.List[str],
+) -> typing.List[typing.Dict[str, str]]:
+    """Verify gathered libraries from invectio."""
+    # Use Thoth user-API endpoint to verify what is the packages using that import name
+    verified_libraries = []
+
+    for import_name in gathered_libraries:
+        unique_packages: typing.List[typing.Dict[str, str]] = []
+
+        try:
+            imported_packages = get_package_from_imported_packages(import_name)
+
+            if imported_packages:
+                for package in imported_packages:
+                    if package["package_name"] not in [p["package_name"] for p in unique_packages]:
+                        unique_packages.append(
+                            {
+                                "package_name": package["package_name"],
+                                "index_url": package["index_url"],
+                            }
+                        )
+                    else:
+                        existing_indexes = [
+                            p["index_url"] for p in unique_packages if p["package_name"] == package["package_name"]
+                        ]
+
+                        if package["index_url"] not in existing_indexes:
+                            unique_packages.append(
+                                {
+                                    "package_name": package["package_name"],
+                                    "index_url": package["index_url"],
+                                }
+                            )
+
+                for unique_package in unique_packages:
+                    verified_libraries.append(unique_package)
+                    _LOGGER.info(
+                        f"Package name {unique_package['package_name']} identifed for import name {import_name}"
+                    )
+
+        except Exception as error:
+            _LOGGER.warning(f"No packages identified for import name {import_name}: {error}")
+            _LOGGER.warning(f"Using {import_name} as package identified.")
+            verified_libraries.append(
+                {
+                    "package_name": import_name,
+                    "index_url": "https://pypi.org/simple",
+                }
+            )
+
+    return verified_libraries
+
+
 def gather_libraries(notebook_path: str) -> typing.List[str]:
     """Gather libraries with invectio."""
-    library_gathered = []
+    gathered_libraries = []
     try:
         notebook_content_py = get_notebook_content(notebook_path=notebook_path, py_format=True)
 
@@ -503,11 +598,11 @@ def gather_libraries(notebook_path: str) -> typing.List[str]:
         std_lib = {p.name.rstrip(".py") for p in std_lib_path.iterdir()}
 
         libs = filter(lambda k: k not in std_lib | set(sys.builtin_module_names), report)
-        library_gathered = list(libs)
+        gathered_libraries = list(libs)
     except Exception as e:
         _LOGGER.error(f"Could not gather libraries: {e}")
 
-    return library_gathered
+    return gathered_libraries
 
 
 def load_files(base_path: str) -> typing.Tuple[str, typing.Optional[str]]:
@@ -517,8 +612,8 @@ def load_files(base_path: str) -> typing.Tuple[str, typing.Optional[str]]:
     pipfile_lock_path = Path(base_path).joinpath("Pipfile.lock")
 
     project = Project.from_files(
-        pipfile_path=pipfile_path,
-        pipfile_lock_path=pipfile_lock_path,
+        pipfile_path=str(pipfile_path),
+        pipfile_lock_path=str(pipfile_lock_path),
         without_pipfile_lock=not pipfile_lock_path.exists(),
     )
 
@@ -545,11 +640,12 @@ def lock_dependencies_with_thoth(
     debug: bool,
     notebook_content: str,
     kernels_path: Path = Path.home().joinpath(".local/share/thoth/kernels"),
-) -> typing.Tuple[int, dict]:
+    labels: typing.Optional[typing.Dict[str, str]] = None,
+) -> typing.Tuple[int, typing.Dict[str, typing.Any]]:
     """Lock dependencies using Thoth resolution engine."""
     initial_path = Path.cwd()
     # Get origin before changing path
-    origin: str = _get_origin()
+    origin: typing.Optional[str] = _get_origin()
     _LOGGER.info("Origin identified by thamos: %r", origin)
 
     env_path = kernels_path.joinpath(kernel_name)
@@ -562,7 +658,16 @@ def lock_dependencies_with_thoth(
     _LOGGER.info("Current path: %r ", env_path.as_posix())
     _LOGGER.info(f"Input Pipfile: \n{pipfile_string}")
 
-    advise = {"requirements": {}, "requirement_lock": {}, "error": False, "error_msg": "", "stack_info": {}}
+    advise = {
+        "thoth_analysis_id": "",
+        "requirements": {},
+        "requirements_lock": {},
+        "error": False,
+        "error_msg": "",
+        "stack_info": [],
+        "justification": [],
+    }
+
     returncode = 0
 
     temp = tempfile.NamedTemporaryFile(prefix="jl_thoth_", mode="w+t")
@@ -587,6 +692,7 @@ def lock_dependencies_with_thoth(
             timeout=timeout,
             src_path=temp.name,
             debug=debug,
+            labels=labels,
         )
 
         _LOGGER.info(f"Response: {response}")
@@ -595,6 +701,8 @@ def lock_dependencies_with_thoth(
             raise Exception("Analysis was not successful.")
 
         result, error_result = response
+        thoth_analysis_id = result["parameters"]["output"].split("/")[-1]
+        advise["thoth_analysis_id"] = thoth_analysis_id
 
         if error_result:
             advise["error"] = True
@@ -605,21 +713,17 @@ def lock_dependencies_with_thoth(
             # Use report of the best one, therefore index 0
             if result["report"] and result["report"]["products"]:
                 justifications = result["report"]["products"][0]["justification"]
-                _LOGGER.info(f"Justification: {justifications}")
-
-                stack_info = result["report"]["stack_info"]
-                _LOGGER.debug(f"Stack info {stack_info}")
-
                 pipfile = result["report"]["products"][0]["project"]["requirements"]
                 pipfile_lock = result["report"]["products"][0]["project"]["requirements_locked"]
 
                 advise["requirements"] = pipfile
-                advise["requirement_lock"] = pipfile_lock
+                advise["requirements_lock"] = pipfile_lock
                 advise["error"] = False
-                advise["justification"] = result["report"]["products"][0]["justification"]
+                advise["justification"] = justifications
 
             if result["report"] and result["report"]["stack_info"]:
-                advise["stack_info"] = result["report"]["stack_info"]
+                stack_info = result["report"]["stack_info"]
+                advise["stack_info"] = stack_info
 
     except Exception as api_error:
         _LOGGER.debug(f"error locking dependencies using Thoth: {api_error}")
@@ -646,7 +750,7 @@ def lock_dependencies_with_thoth(
 
             if requirements_format == "pipenv":
                 _LOGGER.info("Writing to Pipfile/Pipfile.lock in %r", env_path.as_posix())
-                project.to_files(pipfile_path=pipfile_path, pipfile_lock_path=pipfile_lock_path)
+                project.to_files(pipfile_path=str(pipfile_path), pipfile_lock_path=str(pipfile_lock_path))
         except Exception as e:
             _LOGGER.debug("Requirements files cannot be stored due to: %r", e)
 
@@ -668,7 +772,7 @@ def get_thoth_config(
 
     _LOGGER.info(f"kernel_name selected: {kernel_name} and path: {env_path}")
 
-    config = _Configuration()
+    config = _Configuration()  # type: ignore
 
     if not config.config_file_exists():
         _LOGGER.debug("Thoth config does not exist, creating it...")
@@ -688,7 +792,7 @@ def lock_dependencies_with_pipenv(
     kernel_name: str,
     pipfile_string: str,
     kernels_path: Path = Path.home().joinpath(".local/share/thoth/kernels"),
-) -> typing.Tuple[int, dict]:
+) -> typing.Tuple[int, typing.Dict[str, typing.Any]]:
     """Lock dependencies using Pipenv resolution engine."""
     initial_path = Path.cwd()
     env_path = kernels_path.joinpath(kernel_name)
@@ -795,7 +899,7 @@ def horus_requirements_command(
     add: typing.Optional[typing.List[str]] = None,
     remove: typing.Optional[typing.List[str]] = None,
     save_in_notebook: bool = True,
-):
+) -> Pipfile:
     """Horus requirements command."""
     notebook = get_notebook_content(notebook_path=path)
     notebook_metadata = dict(notebook.get("metadata"))
@@ -853,7 +957,7 @@ def horus_requirements_command(
     return pipfile_
 
 
-def create_pipfile_from_packages(packages: list, python_version: str) -> Pipfile:
+def create_pipfile_from_packages(packages: typing.List[str], python_version: str) -> Pipfile:
     """Create Pipfile from list of packages."""
     source = Source(url="https://pypi.org/simple", name="pypi", verify_ssl=True)
 
@@ -870,7 +974,7 @@ def create_pipfile_from_packages(packages: list, python_version: str) -> Pipfile
     return pipfile_
 
 
-def save_notebook_content(notebook_path: str, notebook: dict):
+def save_notebook_content(notebook_path: str, notebook: typing.Dict[str, typing.Any]) -> typing.Dict[str, typing.Any]:
     """Save notebook content."""
     with open(notebook_path, "w") as notebook_content:
         json.dump(notebook, notebook_content)
@@ -883,7 +987,7 @@ def horus_show_command(
     pipfile: bool = False,
     pipfile_lock: bool = False,
     thoth_config: bool = False,
-):
+) -> typing.Dict[str, typing.Any]:
     """Horus show command."""
     show_all: bool = False
 
@@ -894,6 +998,7 @@ def horus_show_command(
     results = {}
     results["kernel_name"] = ""
     results["dependency_resolution_engine"] = ""
+    results["thoth_analysis_id"] = ""
     results["pipfile"] = ""
     results["pipfile_lock"] = ""
     results["thoth_config"] = ""
@@ -918,6 +1023,9 @@ def horus_show_command(
     dependency_resolution_engine = notebook_metadata.get("dependency_resolution_engine")
     results["dependency_resolution_engine"] = dependency_resolution_engine
 
+    thoth_analysis_id = notebook_metadata.get("thoth_analysis_id")
+    results["thoth_analysis_id"] = thoth_analysis_id
+
     pipfile_string = notebook_metadata.get("requirements")
 
     if pipfile or pipfile_lock or show_all:
@@ -950,27 +1058,75 @@ def horus_show_command(
         if not thoth_config_string:
             results["thoth_config"] = "No .thoth.yaml identified in notebook metadata."
         else:
-            config = _Configuration()
+            config = _Configuration()  # type: ignore
             config.load_config_from_string(thoth_config_string)
             results["thoth_config"] = f"\n.thoth.yaml:\n\n{yaml.dump(config.content)}"
 
     return results
 
 
+def update_runtime_environment_in_thoth_config(
+    kernel: str,
+    config: str,
+    os_name: typing.Optional[str] = None,
+    os_version: typing.Optional[str] = None,
+    python_version: typing.Optional[str] = None,
+    recommendation_type: typing.Optional[str] = None,
+) -> str:
+    """Update runtime environment in thoth config."""
+    thoth_config = _Configuration()  # type: ignore
+    thoth_config.load_config_from_string(config)
+
+    runtime_environments = []
+    # runtime environment with same name does not exists.
+    runtime_environment = dict(thoth_config.get_runtime_environment())
+    runtime_environment["name"] = kernel
+
+    # Assign parameters
+    operating_system = {
+        "name": runtime_environment["operating_system"]["name"],
+        "version": runtime_environment["operating_system"]["version"],
+    }
+
+    if os_name:
+        operating_system["name"] = os_name
+
+    if os_version:
+        operating_system["version"] = os_version
+
+    runtime_environment["operating_system"] = operating_system
+
+    if python_version:
+        runtime_environment["python_version"] = python_version
+
+    if recommendation_type:
+        runtime_environment["recommendation_type"] = recommendation_type
+
+    # Assign runtime environment to thoth config runtime environment.
+    runtime_environments.append(runtime_environment)
+    thoth_config.content["runtime_environments"] = runtime_environments
+
+    # TODO: Use adjusted method from thamos
+    # thoth_config.set_runtime_environment(runtime_environment=runtime_environment, force=True)
+
+    return json.dumps(thoth_config.content)
+
+
 def horus_lock_command(
     path: str,
-    resolution_engine: str,
-    timeout: int,
-    force: bool,
-    debug: bool,
-    recommendation_type: str,
+    resolution_engine: str = "thoth",
+    timeout: int = 180,
+    force: bool = False,
+    debug: bool = False,
+    recommendation_type: str = "latest",
     kernel_name: typing.Optional[str] = None,
     os_name: typing.Optional[str] = None,
     os_version: typing.Optional[str] = None,
     python_version: typing.Optional[str] = None,
+    labels: typing.Optional[typing.Dict[str, str]] = None,
     save_in_notebook: bool = True,
     save_on_disk: bool = False,
-):
+) -> typing.Tuple[typing.Dict[str, typing.Any], typing.Dict[str, typing.Any]]:
     """Lock requirements in notebook metadata."""
     results = {}
     results["kernel_name"] = ""
@@ -1014,7 +1170,7 @@ def horus_lock_command(
         if not thoth_config_string:
             thoth_config = get_thoth_config(kernel_name=kernel)
         else:
-            thoth_config = _Configuration()
+            thoth_config = _Configuration()  # type: ignore
             thoth_config.load_config_from_string(thoth_config_string)
 
         try:
@@ -1023,51 +1179,33 @@ def horus_lock_command(
             _LOGGER.error(f"Could not get notebook content!: {e!r}")
             notebook_content_py = ""
 
-        # runtime environment
-        runtime_environments = []
-        runtime_environment = dict(thoth_config.get_runtime_environment())
-        runtime_environment["name"] = kernel
-
-        operating_system = {
-            "name": runtime_environment["operating_system"]["name"],
-            "version": runtime_environment["operating_system"]["version"],
-        }
-
-        if os_name:
-            operating_system["name"] = os_name
-
-        if os_version:
-            operating_system["version"] = os_version
-
-        runtime_environment["operating_system"] = operating_system
-
-        if python_version:
-            runtime_environment["python_version"] = python_version
-
-        runtime_environment["recommendation_type"] = recommendation_type
-
-        runtime_environments.append(runtime_environment)
-
-        # Assign runtime environment to thoth config runtime environment.
-        thoth_config.content["runtime_environments"] = runtime_environments
-
-        # TODO: Use adjusted method from thamos
-        # thoth_config.set_runtime_environment(runtime_environment=runtime_environment, force=True)
+        # update runtime environment in thoth config
+        thoth_config_updated = update_runtime_environment_in_thoth_config(
+            kernel=kernel,
+            config=json.dumps(thoth_config.content),
+            os_name=os_name,
+            os_version=os_version,
+            python_version=python_version,
+            recommendation_type=recommendation_type,
+        )
 
         _, lock_results = lock_dependencies_with_thoth(
             kernel_name=kernel,
             pipfile_string=requirements,
-            config=json.dumps(thoth_config.content),
+            config=thoth_config_updated,
             timeout=timeout,
             force=force,
             debug=debug,
             notebook_content=notebook_content_py,
+            labels=labels,
         )
+        lock_results["thoth_config"] = thoth_config_updated
 
         if not lock_results["error"]:
             requirements = lock_results["requirements"]
-            requirements_lock = lock_results["requirement_lock"]
-            notebook_metadata["thoth_config"] = json.dumps(thoth_config.content)
+            requirements_lock = lock_results["requirements_lock"]
+            notebook_metadata["thoth_config"] = thoth_config_updated
+            notebook_metadata["thoth_analysis_id"] = lock_results["thoth_analysis_id"]
 
         else:
             error = True
@@ -1075,6 +1213,10 @@ def horus_lock_command(
     if resolution_engine == "pipenv":
         _, lock_results = lock_dependencies_with_pipenv(kernel_name=kernel, pipfile_string=pipfile_.to_string())
 
+        # Remove if Pipenv is used after Thoth was used.
+        if notebook_metadata.get("thoth_analysis_id"):
+            notebook_metadata["thoth_analysis_id"] = ""
+
         if not lock_results["error"]:
             requirements_lock = lock_results["requirements_lock"]
         else:
@@ -1094,11 +1236,10 @@ def horus_lock_command(
 
         if resolution_engine == "thoth":
             # thoth
-            thoth_config_string = json.dumps(thoth_config.content)
-            config = _Configuration()
-            config.load_config_from_string(thoth_config_string)
+            config = _Configuration()  # type: ignore
+            config.load_config_from_string(thoth_config_updated)
             config_path = complete_path.joinpath(".thoth.yaml")
-            config.save_config(path=config_path)
+            config.save_config(path=str(config_path))
 
     if save_in_notebook and not error:
         notebook_metadata["dependency_resolution_engine"] = resolution_engine
@@ -1122,7 +1263,7 @@ def horus_set_kernel_command(
     resolution_engine: typing.Optional[str] = None,
     is_magic_command: bool = False,
     force: bool = False,
-):
+) -> typing.Dict[str, typing.Any]:
     """Create kernel using dependencies in notebook metadata."""
     results = {}
     results["kernel_name"] = ""
@@ -1149,7 +1290,7 @@ def horus_set_kernel_command(
     if kernel == "python3":
         kernel = "jupyterlab-requirements"
 
-    results["kernel_name"]: str = kernel
+    results["kernel_name"]: str = kernel  # type: ignore
 
     home = Path.home()
     store_path: Path = home.joinpath(".local/share/thoth/kernels")
@@ -1179,22 +1320,22 @@ def horus_set_kernel_command(
         pipfile_string = notebook_metadata.get("requirements")
         pipfile_ = Pipfile.from_string(pipfile_string)
         pipfile_path = complete_path.joinpath("Pipfile")
-        pipfile_.to_file(path=pipfile_path)
+        pipfile_.to_file(path=str(pipfile_path))
 
     # requirements lock
     if not is_magic_command:
         pipfile_lock_string = notebook_metadata.get("requirements_lock")
         pipfile_lock_ = PipfileLock.from_string(pipfile_content=pipfile_lock_string, pipfile=pipfile_)
         pipfile_lock_path = complete_path.joinpath("Pipfile.lock")
-        pipfile_lock_.to_file(path=pipfile_lock_path)
+        pipfile_lock_.to_file(path=str(pipfile_lock_path))
 
     if dependency_resolution_engine == "thoth" and not is_magic_command:
         # thoth
         thoth_config_string = notebook_metadata.get("thoth_config")
-        config = _Configuration()
+        config = _Configuration()  # type: ignore
         config.load_config_from_string(thoth_config_string)
         config_path = complete_path.joinpath(".thoth.yaml")
-        config.save_config(path=config_path)
+        config.save_config(path=str(config_path))
 
     # 2. Create virtualenv and install dependencies
     install_packages(
@@ -1225,7 +1366,7 @@ def horus_extract_command(
     thoth_config: bool = False,
     use_overlay: bool = False,
     force: bool = False,
-):
+) -> typing.Dict[str, typing.Any]:
     """Horus extract command."""
     results = {}
     results["kernel_name"] = ""
@@ -1287,7 +1428,7 @@ def horus_extract_command(
                 "Use --force to overwrite existing content or --show-only to visualize it."
             )
         else:
-            pipfile_.to_file(path=pipfile_path)
+            pipfile_.to_file(path=str(pipfile_path))
 
     if pipfile_lock or extract_all:
         pipfile_lock_string = notebook_metadata.get("requirements_lock")
@@ -1305,7 +1446,7 @@ def horus_extract_command(
                 "Use --force to overwrite existing content or --show-only to visualize it."
             )
         else:
-            pipfile_lock_.to_file(path=pipfile_lock_path)
+            pipfile_lock_.to_file(path=str(pipfile_lock_path))
 
     if thoth_config or extract_all:
         thoth_config_string = notebook_metadata.get("thoth_config")
@@ -1313,7 +1454,7 @@ def horus_extract_command(
         if not thoth_config_string:
             raise KeyError("No .thoth.yaml identified in notebook metadata.")
 
-        config = _Configuration()
+        config = _Configuration()  # type: ignore
         config.load_config_from_string(thoth_config_string)
 
         yaml_path = Path(".thoth.yaml")
@@ -1326,3 +1467,27 @@ def horus_extract_command(
             config.save_config()
 
     return results
+
+
+def horus_log_command(notebook_path: str) -> str:
+    """Get log analysis results from adviser ID."""
+    notebook = get_notebook_content(notebook_path=notebook_path)
+    notebook_metadata = notebook.get("metadata")
+
+    if "requirements_lock" not in notebook_metadata.keys():
+        raise Exception(f"notebook at {notebook_path} does not has locked requirements.")
+
+    if "dependency_resolution_engine" not in notebook_metadata.keys():
+        raise Exception(f"notebook at {notebook_path} misses resolution engine key in metadata.")
+
+    if notebook_metadata["dependency_resolution_engine"] != "thoth":
+        raise Exception("This command is available only for Thoth resolution engine.")
+
+    if "thoth_analysis_id" not in notebook_metadata.keys():
+        raise Exception(f"thoth_analysis_id key is not present in the metadata of notebook at: {notebook_path}.")
+    else:
+        thoth_analysis_id = notebook_metadata["thoth_analysis_id"]
+
+    log_str = get_log(thoth_analysis_id)
+
+    return log_str  # type: ignore
