@@ -24,8 +24,6 @@ import argparse
 import ipynbname
 
 from pathlib import Path
-from rich.console import Console
-from rich.table import Table
 
 from IPython.core.magic import line_magic  # Called with %
 
@@ -33,9 +31,9 @@ from IPython.core.magic import line_magic  # Called with %
 # from IPython.core.magic import cell_magic  # Called with %%
 from IPython.core.magic import magics_class, Magics
 from thamos.discover import discover_python_version
+from thamos.cli import _parse_labels
 
-from .lib import _EMOJI
-from .lib import _print_report
+from .lib import print_report
 from .lib import horus_check_metadata_content
 from .lib import create_pipfile_from_packages
 from .lib import gather_libraries
@@ -44,20 +42,22 @@ from .lib import get_packages
 from .lib import horus_delete_kernel
 from .lib import horus_extract_command
 from .lib import horus_list_kernels
+from .lib import horus_log_command
 from .lib import horus_requirements_command
 from .lib import horus_set_kernel_command
 from .lib import horus_show_command
 from .lib import horus_lock_command
+from .lib import verify_gathered_libraries
 
 _LOGGER = logging.getLogger("thoth.jupyterlab_requirements.magic_commands")
 
 
 @magics_class
-class HorusMagics(Magics):
+class HorusMagics(Magics):  # type: ignore[misc]
     """Horus jupyterlab-requirements CLI as magic commands."""
 
-    @line_magic
-    def horus(self, line: str):
+    @line_magic  # type: ignore[misc]
+    def horus(self, line: str):  # type: ignore
         """Horus magic commands."""
         parser = argparse.ArgumentParser(description="This is Horus: jupyterlab-requirements CLI.")
         parser.add_argument("--verbose", help="increase output verbosity", action="store_true")
@@ -66,6 +66,9 @@ class HorusMagics(Magics):
         # command: check
         _ = subparsers.add_parser("check", description="Check dependencies in notebook metadata.")
 
+        # command: log
+        _ = subparsers.add_parser("log", description="Check log from Thoth analysis id, if available.")
+
         # command: requirements
         requirements_command = subparsers.add_parser(
             "requirements", description="Add/Remove one or multiple requirements from the notebook."
@@ -139,6 +142,11 @@ class HorusMagics(Magics):
             type=str,
             help="Use Python version for request to Thoth.",
         )
+        lock_command.add_argument(
+            "--labels",
+            type=str,
+            help="Provide labels for the request to Thoth.",
+        )
 
         # Use Pipenv
         lock_command.add_argument("--pipenv", help="Use pipenv resolution engine.", action="store_true")
@@ -265,45 +273,21 @@ class HorusMagics(Magics):
             notebook = get_notebook_content(notebook_path=nb_path)
             notebook_metadata = notebook.get("metadata")
 
-            result = horus_check_metadata_content(notebook_metadata=notebook_metadata, is_cli=False)
+            results = horus_check_metadata_content(notebook_metadata=notebook_metadata, is_cli=False)
 
-            result.insert(
+            results.insert(
                 0,
                 {
-                    "message": f"notebook name: {nb_name}",
+                    "key": "notebook_name",
+                    "message": nb_name,
                     "type": "INFO",
                 },
             )
 
-            table = Table()
-
-            header = set()
-            for item in result:
-                for key in item.keys():
-                    header.add(key)
-
-            header_sorted = sorted(header)
-            for element in header_sorted:
-                table.add_column(
-                    element.replace("_", " ").capitalize(),
-                    style="cyan",
-                    overflow="fold",
-                )
-
-            for item in result:
-                row = []
-                for key in header_sorted:
-                    entry = item.get(key)
-                    if not bool(int(os.getenv("JUPYTERLAB_REQUIREMENTS_NO_EMOJI", 0))) and isinstance(entry, str):
-                        entry = _EMOJI.get(entry, entry)
-
-                    row.append(entry if entry is not None else "-")
-
-                table.add_row(*row)
-
-            console = Console()
-
-            return console.print(table, justify="center")
+            return print_report(
+                results,
+                title="Horus check results",
+            )
 
         if args.command == "requirements":
             _LOGGER.info("Managing requirements in notebook content.")
@@ -322,49 +306,27 @@ class HorusMagics(Magics):
         if args.command == "show":
             _LOGGER.info("Show dependencies content from notebook content.")
 
-            results = horus_show_command(
+            result = horus_show_command(
                 path=nb_path,
                 pipfile=args.pipfile,
                 pipfile_lock=args.pipfile_lock,
                 thoth_config=args.thoth_config,
             )
 
-            result = []
-            for r in results:
+            results = []
+            for r in result:
 
-                result.append(
+                results.append(
                     {
                         "type": r,
-                        "result": results[r],
+                        "result": result[r],
                     }
                 )
 
-            table = Table()
-
-            header = set()
-            for item in result:
-                for key in item.keys():
-                    header.add(key)
-
-            header_sorted = sorted(header)
-            for element in header_sorted:
-                table.add_column(
-                    element.replace("_", " ").capitalize(),
-                    style="cyan",
-                    overflow="fold",
-                )
-
-            for item in result:
-                row = []
-                for key in header_sorted:
-                    entry = item.get(key)
-                    row.append(entry if entry is not None else "-")
-
-                table.add_row(*row)
-
-            console = Console()
-
-            return console.print(table, justify="center")
+            return print_report(
+                results,
+                title="Horus show results",
+            )
 
         if args.command == "convert":
             _LOGGER.info("Cleaning notebook content to allow reproducibility...")
@@ -372,7 +334,7 @@ class HorusMagics(Magics):
         if args.command == "lock":
             _LOGGER.info("Show dependencies content from notebook content.")
 
-            results, lock_results = horus_lock_command(
+            general_results, lock_results = horus_lock_command(
                 path=nb_path,
                 resolution_engine="thoth" if not args.pipenv else "pipenv",
                 timeout=args.timeout,
@@ -382,6 +344,7 @@ class HorusMagics(Magics):
                 kernel_name=args.kernel_name,
                 os_name=args.os_name,
                 os_version=args.os_version,
+                labels=_parse_labels(args.labels),
                 python_version=args.python_version,
                 save_in_notebook=False,
                 save_on_disk=True,
@@ -394,14 +357,14 @@ class HorusMagics(Magics):
 
                 if not args.pipenv:
                     if lock_results["stack_info"]:
-                        _print_report(
+                        print_report(
                             lock_results["stack_info"],
                             title="Application stack guidance",
                         )
 
                     # Print report of the best one - thus index zero.
                     if lock_results["justification"]:
-                        _print_report(
+                        print_report(
                             lock_results["justification"],
                             title="Recommended stack report",
                         )
@@ -410,14 +373,15 @@ class HorusMagics(Magics):
                     path=nb_path,
                     kernel_name=args.kernel_name,
                     save_in_notebook=False,
-                    resolution_engine=results["dependency_resolution_engine"],
+                    resolution_engine=general_results.get("dependency_resolution_engine"),
                     is_magic_command=True,
                 )
 
                 return json.dumps(
                     {
                         "kernel_name": kernel_results["kernel_name"],
-                        "resolution_engine": results["dependency_resolution_engine"],
+                        "resolution_engine": general_results.get("dependency_resolution_engine"),
+                        "thoth_analysis_id": lock_results.get("thoth_analysis_id"),
                     }
                 )
 
@@ -447,38 +411,16 @@ class HorusMagics(Magics):
 
             packages = get_packages(kernel_name=args.kernel_name)
 
-            result = []
+            results = []
 
             for package_name in packages:
 
-                result.append({"package name": package_name, "package version": packages[package_name]})
+                results.append({"package name": package_name, "package version": packages[package_name]})
 
-            table = Table()
-
-            header = set()
-            for item in result:
-                for key in item.keys():
-                    header.add(key)
-
-            header_sorted = sorted(header)
-            for element in header_sorted:
-                table.add_column(
-                    element.replace("_", " ").capitalize(),
-                    style="cyan",
-                    overflow="fold",
-                )
-
-            for item in result:
-                row = []
-                for key in header_sorted:
-                    entry = item.get(key)
-                    row.append(entry if entry is not None else "-")
-
-                table.add_row(*row)
-
-            console = Console()
-
-            return console.print(table, justify="center")
+            return print_report(
+                results,
+                title=f"{args.kernel_name} kernel packages",
+            )
 
         if args.command == "delete-kernel":
             _LOGGER.info("Delete kernel (if exists).")
@@ -502,52 +444,32 @@ class HorusMagics(Magics):
 
         if args.command == "list-kernels":
             kernels = horus_list_kernels()
-            result = []
+            results = []
             for k in kernels:
 
                 if k == "python3":
-                    result.append(
+                    results.append(
                         {
                             "kernel name": f"{k} (default)",
                         }
                     )
                 else:
-                    result.append(
+                    results.append(
                         {
                             "kernel name": k,
                         }
                     )
 
-            table = Table()
-
-            header = set()
-            for item in result:
-                for key in item.keys():
-                    header.add(key)
-
-            header_sorted = sorted(header)
-            for element in header_sorted:
-                table.add_column(
-                    element.replace("_", " ").capitalize(),
-                    style="cyan",
-                    overflow="fold",
-                )
-
-            for item in result:
-                row = []
-                for key in header_sorted:
-                    entry = item.get(key)
-                    row.append(entry if entry is not None else "-")
-
-                table.add_row(*row)
-
-            console = Console()
-
-            return console.print(table, justify="center")
+            return print_report(
+                results,
+                title="Available kernels",
+            )
 
         if args.command == "discover":
             _LOGGER.info("Discover dependencies from notebook content.")
-            packages_ = gather_libraries(notebook_path=nb_path)
+            gathered_libraries = gather_libraries(notebook_path=nb_path)
+            verified_libraries = verify_gathered_libraries(gathered_libraries=gathered_libraries)
+            packages_ = [package["package_name"] for package in verified_libraries]
 
             if packages_:
                 _LOGGER.info(f"Thoth invectio libraries gathered: {json.dumps(packages_)}")
@@ -560,6 +482,17 @@ class HorusMagics(Magics):
 
             return json.dumps({"requirements": pipfile.to_dict(), "force": args.force})
 
+        if args.command == "log":
+            try:
+                log_str = horus_log_command(notebook_path=nb_path)
+                if log_str:
+                    lines = log_str.split("\n")
+                    for line in lines:
+                        print(line)
+
+            except Exception as e:
+                raise Exception(e)
+
         if args.command == "extract":
             _LOGGER.info("Extract dependencies content from notebook content.")
 
