@@ -21,6 +21,7 @@ import os
 import json
 import logging
 
+from typing import Dict, Any
 from pathlib import Path
 from jupyter_server.base.handlers import APIHandler
 from .lib import get_thoth_config
@@ -32,11 +33,11 @@ from thamos.config import _Configuration
 _LOGGER = logging.getLogger("jupyterlab_requirements.thoth_config_handler")
 
 
-class ThothConfigHandler(APIHandler):
+class ThothConfigHandler(APIHandler):  # type: ignore[misc]
     """Thoth config handler for user requirements."""
 
     @web.authenticated
-    def post(self):
+    def post(self):  # type: ignore
         """Retrieve or create Thoth config file."""
         input_data = self.get_json_body()
         kernel_name: str = input_data["kernel_name"]
@@ -49,17 +50,17 @@ class ThothConfigHandler(APIHandler):
         self.finish(json.dumps(thoth_config))
 
     @web.authenticated
-    def put(self):
+    def put(self):  # type: ignore
         """Update Thoth config file."""
         initial_path = Path.cwd()
         input_data = self.get_json_body()
-        new_runtime_environment: str = input_data["runtime_environment"]
+        new_runtime_environment: Dict[str, Any] = input_data["runtime_environment"]
         force: bool = input_data["force"]
         complete_path: str = input_data["complete_path"]
 
         os.chdir(complete_path)
 
-        configuration = _Configuration()
+        configuration = _Configuration()  # type: ignore
 
         if not configuration.config_file_exists():
             _LOGGER.info("Thoth config does not exist, creating it...")
