@@ -20,7 +20,7 @@ import { DependencyManagementSaveButton } from './dependencyManagementSaveButton
 import { DependencyManagementInstallButton } from './dependencyManagementInstallButton'
 import { DependencyManagementNewPackageButton } from './dependencyManagementAddPackageButton';
 
-import { get_python_version, take_notebook_content } from "../notebook";
+import { delete_key_from_notebook_metadata, get_python_version, take_notebook_content } from "../notebook";
 import { Requirements, RequirementsLock } from '../types/requirements';
 
 import { ThothConfig } from '../types/thoth';
@@ -57,6 +57,8 @@ import {
   _handle_total_packages_case,
   _handle_thoth_config
 } from "../helpers";
+import StylizedTextInput from "./StylizedTextInput";
+import StylizedDropdown from "./StylizedDropdown";
 
 /**
  * The class name added to the new package button (CSS).
@@ -64,6 +66,9 @@ import {
 const OK_BUTTON_CLASS = "thoth-ok-button";
 const CONTAINER_BUTTON = "thoth-container-button";
 const CONTAINER_BUTTON_CENTRE = "thoth-container-button-centre";
+const INPUT_FORM_BOX = "thoth-form-box";
+const INPUT_OPTIONS = "thoth-inputs-options";
+const FORM_GRID = "form-grid"
 
 /**
  * Class: Holds properties for DependenciesManagementDialog.
@@ -96,9 +101,13 @@ export interface IDependencyManagementUIState {
   thoth_timeout: number,
   thoth_force: boolean,
   thoth_debug: boolean,
+  os_name: string,
+  os_version: string,
+  python_version: string,
   root_directory: string,
   thoth_resolution_error_msg: string,
-  pipenv_resolution_error_msg: string
+  pipenv_resolution_error_msg: string,
+  labels: string,
 }
 
 /**
@@ -140,7 +149,13 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
             },
             python_version: "3.8",
             recommendation_type: "latest",
-            base_image: ""
+            base_image: "",
+            labels: "",
+            hardware: {
+              cpu_model: null,
+              cpu_family: null,
+              gpu_family: null,
+            }
           }]
         },
         error_msg: undefined,
@@ -148,9 +163,13 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
         thoth_timeout: 180,
         thoth_force: false,
         thoth_debug: true,
+        os_name: "ubi",
+        os_version: "8",
+        python_version: "3.8",
         root_directory: "",
         thoth_resolution_error_msg: undefined,
-        pipenv_resolution_error_msg: undefined
+        pipenv_resolution_error_msg: undefined,
+        labels: ""
       }
 
       this.onStart = this.onStart.bind(this),
@@ -163,14 +182,22 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
       this.deleteRow = this.deleteRow.bind(this),
       this.deleteSavedRow = this.deleteSavedRow.bind(this),
       this.onSave = this.onSave.bind(this),
+      this.lock = this.lock.bind(this),
       this.lock_using_thoth = this.lock_using_thoth.bind(this),
       this.lock_using_pipenv = this.lock_using_pipenv.bind(this),
       this.install = this.install.bind(this),
       this.setKernel = this.setKernel.bind(this),
       this.setKernelName = this.setKernelName.bind(this)
-      this.setRecommendationType = this.setRecommendationType.bind(this)
+      this.changeRecommendationType = this.changeRecommendationType.bind(this)
       this.setTimeout = this.setTimeout.bind(this)
+      this.setPythonVersion = this.setPythonVersion.bind(this)
+      this.setOSName = this.setOSName.bind(this)
+      this.setOSVersion = this.setOSVersion.bind(this)
       this.setRootDirectoryPath = this.setRootDirectoryPath.bind(this)
+      this.setThothLabels = this.setThothLabels.bind(this)
+      this.setResolutionEngine = this.setResolutionEngine.bind(this)
+      this.setDebugParameter = this.setDebugParameter.bind(this)
+      this.setForceParameter = this.setForceParameter.bind(this)
 
       this._model = new KernelModel ( this.props.panel.sessionContext )
     }
@@ -237,44 +264,27 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
       this.setState(ui_state);
     }
 
-    /**
-     * Function: Set recommendation type for thamos advise
-     */
-
-    setRecommendationType(recommendation_type: string) {
-
-      this.setState(
-        {
-          recommendation_type: recommendation_type
-        }
-      );
-    }
-
     /**
      * Function: Change recommendation type for thamos advise
      */
 
-    changeRecommendationType(event: React .ChangeEvent<HTMLInputElement>) {
+    changeRecommendationType(selected: string) {
+      this.setState(
+        {
+          recommendation_type: selected,
+        }
+      );
 
-      const recommendation_type = event.target.value;
-      this.setRecommendationType( recommendation_type )
-  }
+    }
 
     /**
      * Function: Set force for thamos advise
      */
 
-     setForceParameter(thoth_force: string) {
-        if (thoth_force == "true") {
-          var force = true
-        }
-        else {
-          var force = false
-        }
-
+     setForceParameter(event: React.ChangeEvent<HTMLInputElement>) {
         this.setState(
           {
-            thoth_force: force
+            thoth_force: event.target.checked
           }
         );
      }
@@ -284,17 +294,10 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
      * Function: Set debug for thamos advise
      */
 
-     setDebugParameter(thoth_debug: string) {
-      if (thoth_debug == "true") {
-        var debug = true
-      }
-      else {
-        var debug = false
-      }
-
+     setDebugParameter(event: React.ChangeEvent<HTMLInputElement>) {
       this.setState(
         {
-          thoth_debug: debug
+          thoth_debug: event.target.checked
         }
       );
    }
@@ -332,7 +335,8 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
 
       this.setState(
         {
-          thoth_config: initial_thoth_config
+          thoth_config: initial_thoth_config,
+          python_version: thoth_python_version
         }
       );
     }
@@ -351,7 +355,8 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
 
       this.setState(
         {
-          thoth_config: initial_thoth_config
+          thoth_config: initial_thoth_config,
+          os_name: thoth_os_name
         }
       );
     }
@@ -370,12 +375,47 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
 
       this.setState(
         {
-          thoth_config: initial_thoth_config
+          thoth_config: initial_thoth_config,
+          os_version: thoth_os_version
         }
       );
     }
 
 
+    /**
+     * Function: Set Thoth labels (comma separated values)
+     */
+
+    setThothLabels(event: React.ChangeEvent<HTMLInputElement>) {
+
+      var labels = event.target.value
+
+      // Check labels inputs (comma separated) is done on the backend currently!
+      var initial_thoth_config = this.state.thoth_config
+
+      _.set(initial_thoth_config, initial_thoth_config.runtime_environments[0].base_image, labels)
+
+      this.setState(
+        {
+          thoth_config: initial_thoth_config,
+          labels: labels
+        }
+      );
+    }
+
+    /**
+     * Function: Set Resolution engine
+     */
+
+     setResolutionEngine(selected: string) {
+      this.setState(
+        {
+          resolution_engine: selected
+        }
+      );
+
+    }
+
     /**
      * Function: Set root directory where to place overlays and where to find .thoth.yaml
      */
@@ -642,7 +682,10 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
       )
 
       if ( _.get(deleted_case, "action_required") == "relock" ) {
-        await this.lock_using_thoth()
+        // Go to saved state, so user can click install button
+        var ui_state = this.state
+        _.set(ui_state, "status", "saved")
+        await this.setNewState(ui_state);
         return
       }
 
@@ -715,8 +758,28 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
         }
     }
 
+    async lock() {
+
+      var current_state = this.state
+      console.log("before locking", current_state)
+
+      if ( current_state.resolution_engine == "thoth" ) {
+          await this.lock_using_thoth()
+      }
+
+      else if ( current_state.resolution_engine == "pipenv" ) {
+        await this.lock_using_pipenv()
+      }
+
+      else {
+          console.error("This resolution engine is not supported: ", current_state.resolution_engine)
+      }
+
+    }
+
     async lock_using_thoth() {
 
+      // Prepare inputs for advise
       var ui_state = this.state
 
       _.set(ui_state, "status", "locking_requirements")
@@ -739,6 +802,7 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
         var thoth_config: ThothConfig = _.get(result, "thoth_config")
       }
 
+      // Run adviser to lock dependencies
       try {
 
         var advise: Advise | undefined = await lock_requirements_with_thoth(
@@ -746,6 +810,11 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
           this.state.thoth_timeout,
           this.state.thoth_force,
           this.state.thoth_debug,
+          this.state.python_version,
+          this.state.os_name,
+          this.state.os_version,
+          this.state.recommendation_type,
+          this.state.labels,
           notebook_content,
           JSON.stringify(thoth_config),
           JSON.stringify(this.state.requirements)
@@ -754,42 +823,84 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
 
       } catch ( error ) {
 
+        // Error in the code
         console.debug("Error locking requirements with Thoth", error)
-        _.set(ui_state, "status", "locking_requirements_using_pipenv")
+        _.set(ui_state, "status", "failed_re_thoth")
+        _.set(ui_state, "thoth_resolution_error_msg", "Error trying to lock dependencies with Thoth" )
+        _.set(
+          ui_state,
+          "error_msg",
+          error
+        )
         await this.setNewState(ui_state);
         return
       }
 
-      if ( advise == undefined || advise.error == true ) {
-        if ( advise != undefined ) {
+      console.log(advise)
+
+      // Error in the resolution process
+      if ( advise == undefined ) {
+        var error_msg = "Thoth resolution engine was not able to lock dependencies, try Pipenv or please contact Thoth team: "
+        _.set(ui_state, "status", "failed_re_thoth")
+        _.set(
+          ui_state,
+          "error_msg",
+          error_msg
+        )
+        await this.setNewState(ui_state);
+        return
+      }
+
+      if ( advise.error != true && advise.error != false && advise.error ) {
+          var error_msg = "Check error and try again."
+          console.log("Thoth resolution engine error:", advise.error)
+          _.set(ui_state, "thoth_resolution_error_msg", advise.error )
+          _.set(ui_state, "status", "failed_re_thoth")
+          _.set(
+            ui_state,
+            "error_msg",
+            error_msg
+          )
+          await this.setNewState(ui_state);
+          return
+      }
+
+      if ( advise.error == true ) {
+          var error_msg = "Thoth resolution engine was not able to lock dependencies, try Pipenv or please contact Thoth team: "
           console.log("Thoth resolution engine error:", advise.error_msg)
-        }
-        _.set(ui_state, "thoth_resolution_error_msg", advise.error_msg )
-        _.set(ui_state, "resolution_engine", "pipenv" )
-        _.set(ui_state, "status", "locking_requirements_using_pipenv")
+          _.set(ui_state, "thoth_resolution_error_msg", advise.error_msg )
+        _.set(ui_state, "status", "failed_re_thoth")
+        _.set(
+          ui_state,
+          "error_msg",
+          error_msg
+        )
         await this.setNewState(ui_state);
         return
       }
 
+      // Resolution process succeeded, saving all data.
       try {
 
         await set_notebook_metadata(
           this.props.panel,
           "thoth",
           advise.requirements,
-          advise.requirement_lock,
-          this.state.thoth_config
+          advise.requirements_lock,
+          this.state.thoth_config,
+          advise.thoth_analysis_id
         )
 
       } catch ( error ) {
         console.debug("Error updating notebook metadata", error)
       }
 
+      console.log(advise)
       try {
         await store_dependencies_on_disk(
           this.state.kernel_name,
           advise.requirements,
-          advise.requirement_lock,
+          advise.requirements_lock,
           'overlays',
           this.state.root_directory
         )
@@ -824,9 +935,13 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
 
       var ui_state = this.state
 
+      _.set(ui_state, "status", "locking_requirements_using_pipenv")
+      await this.setNewState(ui_state);
+
       const notebookMetadataRequirements = this.state.requirements;
       console.debug("Requirements for pipenv", JSON.stringify(notebookMetadataRequirements));
 
+      // Run Pipenv to lock dependencies
       try {
 
         var result = await lock_requirements_with_pipenv(
@@ -839,6 +954,7 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
 
         console.log("Error locking requirements with pipenv", error)
 
+        // Error in JH on cluster
         if ( error.message == "Gateway error while locking dependencies with pipenv.") {
             INotification.warning("Task was cancelled due to gateway error, try again please.")
             _.set(ui_state, "status", "failed")
@@ -847,22 +963,23 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
             return
         }
 
-        _.set(ui_state, "status", "failed_re")
+        // Error in code
+        _.set(ui_state, "status", "failed_re_pipenv")
         _.set(ui_state, "error_msg", "Error locking requirements, check logs and please contact Thoth team: ")
         await this.setNewState(ui_state);
         return
       }
 
+      // Error in Pipenv resolution process
       if ( result == undefined || result.error == true ) {
 
-        var error_msg = "No resolution engine was able to install dependencies, please contact Thoth team: "
+        var error_msg = `Pipenv resolution engine was not able to lock dependencies, try Thoth or please contact Thoth team: `
+
         if ( result != undefined ) {
           console.debug("Pipenv resolution engine error:", result.error_msg)
           _.set(ui_state, "pipenv_resolution_error_msg", result.error_msg.toString())
-          var error_msg = `No resolution engine was able to install dependencies! Please contact Thoth team: `
         }
-
-        _.set(ui_state, "status", "failed_re")
+        _.set(ui_state, "status", "failed_re_pipenv")
         _.set(
           ui_state,
           "error_msg",
@@ -872,18 +989,18 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
         return
       }
 
+      // Resolution process succeeded, saving all data.
       try {
-        await set_notebook_metadata(
-          this.props.panel,
-          "pipenv",
-          notebookMetadataRequirements,
-          result.requirements_lock
-        )
+          await set_notebook_metadata(
+            this.props.panel,
+            "pipenv",
+            notebookMetadataRequirements,
+            result.requirements_lock
+          )
+          await delete_key_from_notebook_metadata( this.props.panel, "thoth_analysis_id" )
 
       } catch ( error ) {
-
         console.debug("Error storing metadata in notebook", error)
-
       }
 
       try {
@@ -927,6 +1044,12 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
         this.props.loaded_resolution_engine
       )
 
+      if ( _.has(ui_on_start_state.thoth_config, "runtime_environments") ) {
+        _.set(ui_on_start_state, "os_name", ui_on_start_state.thoth_config.runtime_environments[0].operating_system.name)
+        _.set(ui_on_start_state, "os_version", ui_on_start_state.thoth_config.runtime_environments[0].operating_system.version)
+        _.set(ui_on_start_state, "python_version", ui_on_start_state.thoth_config.runtime_environments[0].python_version)
+      }
+
       _.set(ui_on_start_state, "root_directory", root_directory)
 
       await this.setNewState(ui_on_start_state);
@@ -957,7 +1080,7 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
                                         <div className={CONTAINER_BUTTON}>
                                           <div className={CONTAINER_BUTTON_CENTRE}>
                                             <DependencyManagementInstallButton
-                                            install={this.lock_using_thoth} />
+                                            install={this.lock} />
                                           </div>
                                         </div>
                                       </div>
@@ -976,174 +1099,155 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
                                       </div>
                                     </div>
 
+      let resolutionEngineInput = <StylizedDropdown label="Resolution Engine"
+                                                    value={this.state.resolution_engine}
+                                                    options={["thoth", "pipenv"]}
+                                                    onChange={this.setResolutionEngine}
+                                                    isRequired={true}
+                                                    style={{gridColumn: "span 4"}} />
 
-      let kernelNameInput = <div>
-                              <label>
-                                Kernel name:
-                                <input
-                                  title="Kernel name"
-                                  type="text"
-                                  name="kernel_name"
-                                  value={this.state.kernel_name}
-                                  onChange={this.setKernelName}
-                                />
-                              </label>
-                              <br />
-                            </div>
+      let kernelNameInput = <StylizedTextInput label="Kernel Name"
+                                               value={this.state.kernel_name}
+                                               onChange={this.setKernelName}
+                                               style={{gridColumn: "span 4"}} />
 
-      let pathRootProjectInput =  <div>
-                                    <label>
-                                      Path root project:
-                                      <input
-                                        title="Path root project"
-                                        type="text"
-                                        name="root_directory"
-                                        value={this.state.root_directory}
-                                        onChange={this.setRootDirectoryPath}
-                                      />
-                                    </label>
-                                    <br />
+      let pathRootProjectInput =  <StylizedTextInput label="Path root project"
+                                                     value={this.state.root_directory}
+                                                     onChange={this.setRootDirectoryPath}
+                                                     style={{gridColumn: "span 4"}} />
+
+      let recommendationTypeInput = <StylizedDropdown label="Thoth Recommendation type"
+                                                      value={this.state.recommendation_type}
+                                                      options={["latest", "performance", "security", "stable"]}
+                                                      onChange={this.changeRecommendationType}
+                                                      style={{gridColumn: "span 2"}}/>
+
+      let baseImageInput;
+      if ( this.state.thoth_config ) {
+          if ( this.state.thoth_config.runtime_environments[0].base_image ) {
+            baseImageInput = <StylizedTextInput label="Thoth base image"
+                                                value={this.state.thoth_config.runtime_environments[0].base_image}
+                                                onChange={this.setBaseImage}
+                                                style={{gridColumn: "span 2"}}/>
+          }
+      }
+
+      let timeoutInput = <StylizedTextInput label="Thoth timeout"
+                                            value={this.state.thoth_timeout}
+                                            onChange={this.setTimeout}
+                                            suffix="seconds"
+                                            helperText="0 - 300 seconds"
+                                            style={{gridColumn: "span 2"}} />
+
+      let pythonVersionInput = <StylizedTextInput label="Thoth Python Version"
+                                                  value={this.state.python_version}
+                                                  onChange={this.setPythonVersion}
+                                                  style={{gridColumn: "span 2"}}/>
+
+      let OSNameInput = <StylizedTextInput label="Thoth Operating System Name"
+                                           value={this.state.os_name}
+                                           onChange={this.setOSName}
+                                           style={{gridColumn: "span 2"}}/>
+
+      let OSVersionInput = <StylizedTextInput label="Thoth Operating System Version"
+                                              value={this.state.os_version}
+                                              onChange={this.setOSVersion}
+                                              style={{gridColumn: "span 2"}}/>
+
+      let forceInput =  <div className={INPUT_OPTIONS} style={{gridColumn: "span 1"}}>
+                          <label>
+                            Thoth force
+                            <input type="checkbox"
+                                   title="Thoth force parameter."
+                                   name="thoth_force"
+                                   id="thoth_force"
+                                   onChange={this.setForceParameter}
+                                   checked={this.state.thoth_force} />
+                            </label>
+                          <br />
+                        </div>
+
+      let debugInput =  <div className={INPUT_OPTIONS} style={{gridColumn: "span 1"}}>
+                          <label>
+                            Thoth debug
+                            <input type="checkbox"
+                                   title="Thoth debug parameter."
+                                   name="thoth_debug"
+                                   id="thoth_debug"
+                                   onChange={this.setDebugParameter}
+                                   checked={this.state.thoth_debug} />
+                          </label>
+                          <br />
+                        </div>
+
+      let labelsInput =  <StylizedTextInput label="Labels"
+                                            value={this.state.labels}
+                                            onChange={this.setThothLabels}
+                                            helperText="comma separated labels"
+                                            style={{gridColumn: "span 4"}}/>
+
+      if ( this.state.resolution_engine == "thoth" ) {
+
+        var optionsForm = <div>
+                            <section>
+                              <h2>OPTIONS</h2>
+                            </section>
+
+                            <div className={INPUT_FORM_BOX}>
+                              <form className={FORM_GRID}>
+                                {resolutionEngineInput}
+                                {kernelNameInput}
+                                {pathRootProjectInput}
+                                <div style={{gridColumn: "span 2"}}>
+                                  <p>Python Settings</p>
+                                  <div className="form-stack">
+                                    {recommendationTypeInput}
+                                    {pythonVersionInput}
                                   </div>
+                                </div>
 
-      let recommendationTypeInput = <div>
-                                      <label>
-                                        Thoth Recommendation type:
-                                        <select onChange={() => this.changeRecommendationType}>
-                                          title="Recommendation Type"
-                                          name="recommendation_type"
-                                          value={this.state.recommendation_type}
-                                          <option value="latest">latest</option>
-                                          <option value="performance">performance</option>
-                                          <option value="security">security</option>
-                                          <option value="stable">stable</option>
-                                        </select>
-                                      </label>
-                                      <br />
-                                    </div>
+                                <div style={{gridColumn: "span 2"}}>
+                                  <p>Operating System Settings</p>
+                                  <div className="form-stack">
+                                    {OSNameInput}
+                                    {OSVersionInput}
+                                  </div>
+                                </div>
 
-      if ( this.state.thoth_config.runtime_environments[0].base_image ) {
-        var baseImageInput = <div>
-                              <label>
-                                Thoth base image:
-                                <input
-                                  title="Thoth base image"
-                                  type="text"
-                                  name="thoth_base_image"
-                                  value={this.state.thoth_config.runtime_environments[0].base_image}
-                                  onChange={this.setBaseImage}
-                                />
-                              </label>
-                              <br />
+                                <div style={{gridColumn: "span 4"}}>
+                                  <p>Thoth Advise Settings</p>
+                                  <div className="form-grid">
+                                    {timeoutInput}
+                                    {debugInput}
+                                    {forceInput}
+                                    {labelsInput}
+                                    {baseImageInput}
+                                  </div>
+                                </div>
+                              </form>
                             </div>
+                          </div>
+
       }
 
       else {
-        var baseImageInput = <div></div>
-      }
 
-      let timeoutInput = <div>
-                            <label>
-                              Thoth timeout [s]:
-                              <input
-                                title="Thoth timeout"
-                                type="text"
-                                name="thoth_timeout"
-                                value={this.state.thoth_timeout}
-                                onChange={this.setTimeout}
-                              />
-                            </label>
-                            <br />
-                          </div>
-
-      let pythonVersionInput = <div>
-                                <label>
-                                  Thoth Python Version:
-                                  <input
-                                    title="Thoth OS name"
-                                    type="text"
-                                    name="thoth_python_version"
-                                    value={this.state.thoth_config.runtime_environments[0].python_version}
-                                    onChange={this.setPythonVersion}
-                                  />
-                                </label>
-                                <br />
-                              </div>
-
-      let OSNameInput = <div>
-                          <label>
-                            Thoth OS name:
-                            <input
-                              title="Thoth OS name"
-                              type="text"
-                              name="thoth_os_name"
-                              value={this.state.thoth_config.runtime_environments[0].operating_system.name}
-                              onChange={this.setOSName}
-                            />
-                          </label>
-                          <br />
-                        </div>
-
-      let OSVersionInput = <div>
-                          <label>
-                            Thoth OS version:
-                            <input
-                              title="Thoth OS version"
-                              type="text"
-                              name="thoth_os_version"
-                              value={this.state.thoth_config.runtime_environments[0].operating_system.version}
-                              onChange={this.setOSVersion}
-                            />
-                          </label>
-                          <br />
-                          </div>
-
-      let forceInput =  <div>
-                          <label>
-                            Thoth force:
-                            <select onChange={() => this.setForceParameter}>
-                              title="Thoth force parameter"
-                              name="thoth_force"
-                              value={this.state.thoth_force}
-                              <option value="false">False</option>
-                              <option value="true">True</option>
-                            </select>
-                          </label>
-                          <br />
-                        </div>
-
-      let debugInput =  <div>
-                          <label>
-                            Thoth debug:
-                            <select onChange={() => this.setForceParameter}>
-                              title="Thoth debug parameter"
-                              name="thoth_debug"
-                              value={this.state.thoth_debug}
-                              <option value="false">False</option>
-                              <option value="true">True</option>
-                            </select>
-                          </label>
-                        </div>
-
-
-      let optionsForm = <div>
+        var optionsForm = <div>
                           <section>
                             <h2>OPTIONS</h2>
                           </section>
 
-                          <form>
+                          <div className={INPUT_FORM_BOX}>
+                          <form className={FORM_GRID}>
+                            {resolutionEngineInput}
                             {kernelNameInput}
                             {pathRootProjectInput}
-                            {recommendationTypeInput}
-                            {pythonVersionInput}
-                            {OSNameInput}
-                            {OSVersionInput}
-                            {baseImageInput}
-                            {timeoutInput}
-                            {forceInput}
-                            {debugInput}
                           </form>
+                          </div>
                         </div>
 
+      }
+
       var ui_status = this.state.status
 
       switch(ui_status) {
@@ -1227,8 +1331,8 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
               <div>
                 <fieldset>
                   <p>Pinned down software stack found in notebook metadata!<br></br>
-                  The runtime environment found in the notebook does not match your environment. <br></br>
-                  Please use install button.</p>
+                  The runtime environment found in the notebook does not match the environment in system. <br></br>
+                  Use install button.</p>
                 </fieldset>
               </div>
               {optionsForm}
@@ -1245,7 +1349,7 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
                 <fieldset>
                   <p>No dependencies found in notebook metadata!<br></br>
                   Thoth identified the packages that are required to run this notebook from your code. <br></br>
-                  (Please verify the names of the packages, in the future this won't be necessary). <br></br>
+                  (Names of the packages identified from Thoth database). <br></br>
                   Please use install button to install dependencies identified.</p>
                 </fieldset>
               </div>
@@ -1345,20 +1449,18 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
 
         case "locking_requirements_using_pipenv":
 
-          this.lock_using_pipenv()
           return (
             <div>
               <fieldset>
-                <p>Thoth resolution engine failed... pipenv will be used to lock dependencies!</p>
+                <p>Using Pipenv to lock dependencies...</p>
               </fieldset>
             </div>
         );
 
-        case "failed_re":
+        case "failed_re_thoth":
 
           let paragrah_failure = <p>
             Thoth resolution engine failed because of: {this.state.thoth_resolution_error_msg} <br></br> <br></br>
-              Pipenv resolution engine failed because of: {this.state.pipenv_resolution_error_msg}<br></br> <br></br>
               {this.state.error_msg} <br></br>
           </p>
 
@@ -1394,6 +1496,45 @@ export class DependenciesManagementUI extends React.Component<IDependencyManagem
             </div>
         );
 
+        case "failed_re_pipenv":
+
+          let paragrah_failure_pipenv = <p>
+              Pipenv resolution engine failed because of: {this.state.pipenv_resolution_error_msg}<br></br> <br></br>
+              {this.state.error_msg} <br></br>
+          </p>
+
+          return (
+            <div>
+              <div>
+                <div className={CONTAINER_BUTTON}>
+                  <div className={CONTAINER_BUTTON_CENTRE}>
+                    <button
+                      title='Finish.'
+                      className={OK_BUTTON_CLASS}
+                      onClick={() => this.changeUIstate(
+                        "loading",
+                        this.state.packages,
+                        this.state.loaded_packages,
+                        this.state.installed_packages,
+                        this.state.deleted_packages,
+                        this.state.requirements,
+                        this.state.kernel_name,
+                        this.state.thoth_config,
+                      )
+                      }
+                      >
+                      Ok
+                    </button>
+                  </div>
+                </div>
+              </div>
+              <div>
+                    {paragrah_failure_pipenv}
+              </div>
+              <a href={"https://github.com/thoth-station/jupyterlab-requirements/issues/new?assignees=&labels=bug&template=bug_report.md"} target="_blank"> Open Issue </a>
+            </div>
+        );
+
         case "failed":
 
           return (
