@@ -66,6 +66,11 @@ export async function lock_requirements_with_thoth(
   thoth_timeout: number,
   thoth_force: boolean,
   thoth_debug: boolean,
+  python_version: string,
+  os_name: string,
+  os_version: string,
+  recommendation_type: string,
+  labels: string,
   notebook_content: string,
   thoth_config: string,
   requirements: string,
@@ -77,6 +82,11 @@ export async function lock_requirements_with_thoth(
       thoth_timeout: thoth_timeout,
       thoth_force: thoth_force,
       thoth_debug: thoth_debug,
+      python_version: python_version,
+      os_name: os_name,
+      os_version: os_version,
+      recommendation_type: recommendation_type,
+      labels: labels,
       notebook_content: notebook_content,
       thoth_config: thoth_config,
       requirements: requirements
